/*-
 * Copyright (c) 2004 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_pthread.c,v 1.4 2004/12/19 00:45:58 nork Exp $
 */

#include "pluginwrapper.h"
#include "linux.h"
#include <stddef.h>

#define	PTHREAD_MUTEX_ERRORCHECK	1
typedef	struct pthread_mutex		*pthread_mutex_t;
typedef	struct pthread_mutexattr	*pthread_mutexattr_t;

/*
 * pthread_mutex_init(3)
 */
int __pthread_mutex_init(pthread_mutex_t *, const pthread_mutexattr_t *);

#pragma weak __pthread_mutex_init

int
pthread_mutex_init(linux_pthread_mutex_t *l_mutex, const linux_pthread_mutexattr_t *l_attr)
{
	int ret;
	pthread_mutexattr_t attr;

	pthread_mutexattr_init(&attr);
	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);

	if(  __pthread_mutex_init != NULL  )
		ret = __pthread_mutex_init((pthread_mutex_t *)l_mutex, &attr);
	else
		ret = _pthread_mutex_init((pthread_mutex_t *)l_mutex, &attr);

	pthread_mutexattr_destroy(&attr);

	dprintf("pthread_mutex_init(%08x, %08x[0x%x]) = %d", l_mutex, l_attr, *(int*)l_attr, ret);

	return ret;
}

#if 0
/*
 * pthread_mutex_lock(3)
 */
int _pthread_mutex_lock(pthread_mutex_t *);

int
pthread_mutex_lock(linux_pthread_mutex_t *l_mutex)
{
	int ret;

	ret = _pthread_mutex_lock((pthread_mutex_t *)l_mutex);

	dprintf("pthread_mutex_lock  (%08x) = %d", l_mutex, ret);

	return ret;
}

/*
 * pthread_mutex_unlock(3)
 */
int _pthread_mutex_unlock(pthread_mutex_t *);

int
pthread_mutex_unlock(linux_pthread_mutex_t *l_mutex)
{
	int ret;

	ret = _pthread_mutex_unlock((pthread_mutex_t *)l_mutex);

	dprintf("pthread_mutex_unlock(%08x) = %d", l_mutex, ret);

	return ret;
}
#endif
