/*-
 * Copyright (c) 2002-2003 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: linux_ioctl.c,v 1.3 2003/12/18 18:00:00 nork Exp $
 */

#include "pluginwrapper.h"
#include "linux.h"
#include <errno.h>
#include <stdarg.h>
#include <sys/soundcard.h>


/*
 *
 */
extern int _ioctl(int, unsigned long, ...);

int
ioctl(int d, unsigned long l_request, ...)
{
	int		ret;
	va_list		ap;

	va_start(ap, l_request);

	CLEAR_ERRNO;

	switch(  l_request  )  {

#define	SND_IOCTL_CALL(BSDCTL, LINUXCTL, ...)							\
	case	LINUXCTL:									\
		ret = _ioctl(d, BSDCTL ,##__VA_ARGS__);						\
		dprintf("ioctl(%d, " #LINUXCTL ", ...) = %d / ERRNO = %d", d, ret, errno);	\
		break

	SND_IOCTL_CALL(SNDCTL_DSP_RESET,            LINUX_SNDCTL_DSP_RESET                                                );
	SND_IOCTL_CALL(SNDCTL_DSP_SYNC,             LINUX_SNDCTL_DSP_SYNC                                                 );
	SND_IOCTL_CALL(SNDCTL_DSP_SETFMT,           LINUX_SNDCTL_DSP_SETFMT,            va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_STEREO,           LINUX_SNDCTL_DSP_STEREO,            va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_SPEED,            LINUX_SNDCTL_DSP_SPEED,             va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_SETFRAGMENT,      LINUX_SNDCTL_DSP_SETFRAGMENT,       va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_GETFMTS,          LINUX_SNDCTL_DSP_GETFMTS,           va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_GETCAPS,          LINUX_SNDCTL_DSP_GETCAPS,           va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SNDCTL_DSP_GETOPTR,          LINUX_SNDCTL_DSP_GETOPTR,           va_arg(ap, linux_count_info*)     );
	SND_IOCTL_CALL(SNDCTL_DSP_GETOSPACE,        LINUX_SNDCTL_DSP_GETOSPACE,         va_arg(ap, linux_audio_buf_info*) );
	SND_IOCTL_CALL(SOUND_MIXER_READ_DEVMASK,    LINUX_SOUND_MIXER_READ_DEVMASK,     va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SOUND_PCM_WRITE_CHANNELS,    LINUX_SOUND_PCM_WRITE_CHANNELS,     va_arg(ap, int*)                  );
	SND_IOCTL_CALL(SOUND_PCM_READ_CHANNELS,     LINUX_SOUND_PCM_READ_CHANNELS,      va_arg(ap, int*)                  );

#undef	SND_IOCTL_CALL

	default:
		errno = ENOSYS;
		ret   = -1;
		dprintf("ioctl(fd=%d, request=%08X(on Linux), ...) = %d / ERRNO = %d", d, l_request, ret, errno);
		break;
	}

	va_end(ap);

	return ret;
}
